"use strict";
/**
 * @athenna/core
 *
 * (c) João Lenon <lenon@athenna.io>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AthennaFactory = void 0;
const logger_1 = require("@athenna/logger");
const path_1 = require("path");
const config_1 = require("@athenna/config");
const utils_1 = require("@secjs/utils");
class AthennaFactory {
    constructor(fileName) {
        AthennaFactory.resolveNodeTs(fileName);
        const secConfig = new utils_1.Config();
        secConfig.load(utils_1.Path.config(`app${AthennaFactory.extension}`));
        process.env.NODE_ENV = utils_1.Config.get('app.environment');
        console.log(process.env.NODE_ENV);
        config_1.resolveEnvFile();
        secConfig.load(utils_1.Path.config(`app${AthennaFactory.extension}`));
        Config.load(utils_1.Path.config());
        AthennaFactory.logger = new logger_1.Logger().channel('application', {
            formatterConfig: {
                context: AthennaFactory.name,
            },
        });
        AthennaFactory.registerProviders();
        AthennaFactory.bootProviders();
        AthennaFactory.preloadFiles();
    }
    static bootProviders() {
        const providers = Config.get('app.providers');
        providers.forEach(Provider => {
            if (utils_1.Is.Class(Provider)) {
                AthennaFactory.logger.log(`Booting ${Provider.name}`);
                return new Provider().boot();
            }
            if (utils_1.Is.Object(Provider) && !Provider.default) {
                const firstProviderKey = Object.keys(Provider)[0];
                AthennaFactory.logger.log(`Booting ${firstProviderKey}`);
                return new Provider[firstProviderKey]().boot();
            }
            AthennaFactory.logger.log(`Booting ${Provider.default.name}`);
            // eslint-disable-next-line new-cap
            return new Provider.default().boot();
        });
    }
    static registerProviders() {
        const providers = Config.get('app.providers');
        providers.forEach(Provider => {
            if (utils_1.Is.Class(Provider)) {
                AthennaFactory.logger.log(`Registering ${Provider.name}`);
                return new Provider().register();
            }
            if (utils_1.Is.Object(Provider) && !Provider.default) {
                const firstProviderKey = Object.keys(Provider)[0];
                AthennaFactory.logger.log(`Registering ${firstProviderKey}`);
                return new Provider[firstProviderKey]().register();
            }
            AthennaFactory.logger.log(`Registering ${Provider.default.name}`);
            // eslint-disable-next-line new-cap
            return new Provider.default().register();
        });
    }
    static preloadFiles() {
        const preloads = Config.get('app.preloads');
        preloads.forEach(preload => {
            preload = path_1.normalize(preload);
            const { dir, name } = path_1.parse(utils_1.Path.pwd(preload));
            AthennaFactory.logger.log(`Preloading ${preload} file`);
            require(`${dir}/${name}${this.extension}`);
        });
    }
    async http() {
        const http = ioc.use('Athenna/Core/HttpServer');
        const route = ioc.use('Athenna/Core/HttpRoute');
        route.register();
        const port = Config.get('app.port');
        const host = Config.get('app.host');
        await http.listen(port, host);
        AthennaFactory.logger.log(`Http server started on http://${host}:${port}`);
        return http;
    }
    static resolveNodeTs(fileName) {
        const { ext } = path_1.parse(fileName);
        if (ext === '.ts')
            process.env.NODE_TS = 'true';
        else
            process.env.NODE_TS = 'false';
        AthennaFactory.extension = ext;
    }
}
exports.AthennaFactory = AthennaFactory;
